==> dancingparticles0.2.tgz: dancingparticles0.2/etoile.cc <==
#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include "etoile.h"
#include <gtk/gtk.h>
#include <cstdio>
#include <cstring>

Rect	windRect;

efloatPoint pts[ptsNumMax+10];
efloatPoint speed[ptsNumMax+10];
efloatPoint Center(200.,200.,0);
short ptsNum=ptsNumMin;

gint16 *gfrq[256];
int beat;
int beatcnt;
int thick, quiet;
void loadepic(char *file);

int numpart=6;


efloatPoint Centers[ptsNumMax];
int newline[ptsNumMax];

int numCenters = sizeof(Centers)/sizeof(Centers[0]);
parameters p=ps[0];

void DrawProg(int i,int n);
void Tourne(short i);
void Elastantig(short i,efloatPoint &c);

matrix rot;
static int frames =0;
extern GLfloat heights[64];

void changep()
{
  int i;
  int tot=0;
  int r;
  for(i=0;i<nump;i++)
    tot = tot + ps[i].chance;
  r=rand()%tot;
  for(i=-1;r>=0;r-=ps[i].chance)i++;
  p = ps[i];
    
  if(p.mode==2)
    {
      for(int i=0;i<64;i++)
	Centers[i]= efloatPoint( -400 + i*800/64,0,0);
      numCenters=64;
	    
    }
  if(p.mode==3)
    {
      loadepic("dance.epic");
    }
  if(p.mode==4)
    {
      loadepic("xmms.epic");
    }
}

void etoileinit(void)
{
	
  frames=0;
  pts[0]=Center;
  changep();
  for(int i=1;i<ptsNum;i++)
    {
      pts[i][0]=rand()%100;
      pts[i][1]=rand()%100;
      pts[i][2]=rand()%100;
      //pts[i]=pts[i-1]+diff; 
    }
  cout<<ptsNum<<endl;
  return ;	
}


void etoileLoop(void)
{
	int frames2=frames;
	frames++;
	if(beat==2  && beatcnt%4)
	  {
	    changep();
	    beat--;
	  }
	Center=eSetCenter(frames);

	rot=matrix(p.rotspeed1,efloatPoint(cos(((double)frames2)/120.)*2,cos(((double)frames2)/54.)*2,cos(((double)frames2)/231.)*2));

	for(int i=0;i<ptsNum;i++)
	{
	  switch(p.mode)
	    {
	    case 1:
	      Elastantig(i,Center);
	      Tourne(i);
	      break;
	    case 2:
	    case 3:
	    default:
	      Elastantig(i,Centers[i%numCenters]);
	      Tourne(i);
	    }
	}
	
	BeginDrawing();
	for(int i=0;i<ptsNum;i++)
	  DrawPt(i);	
	EndDrawing();
	
}
void Elastantig(short i , efloatPoint & ctr)
{
  efloatPoint dist = pts[i]-ctr;
  float d=(float)dist.length();
  efloatPoint vectunite=(dist / d);
  
  for(int j=0;j<ptsNum;j++)
    {
      if(i!=j)
	{
	  efloatPoint vect=pts[i]-pts[j];
	  float d1=(float)vect.length();
	  float d2=d1;
	  if(d2!=0)
	    {
	      // d include the normalisation
	      for(int k=0; k< p.antigorder;k++)
		d2=d2*d1;
	      d2=p.ag/d2;
	      
	      if(d2>p.maxantig)
		d2=p.maxantig;
	      vect=(vect * d2);
	      if(p.noagexplosion)
		vect= vect - ( vectunite * (vect % vectunite));

	      speed[i]+= vect ;
	    }
	  else
	    {
	      pts[i][0]+=0.01;
	    }
	}
    }

  float d0=p.d0min+ (((i +frames)%ptsNum<p.dancingpart*ptsNum)?
		     (heights[i%p.numfrq]*p.velocity)
		     :0);
  float f=-p.k*(d-d0);
  vectunite = vectunite* f;
  speed[i]+= vectunite ;
  

  speed[i]= speed[i]  /p.visc;
  pts[i]+= speed[i];// /1000;

}

void Tourne2(short i)
{
/*	ApplyMatrix(pts[i],
			1, 0, 3E-2,
			0, 1 , 1.99461761555E-2,
			-1.99461761555E-2, -2.03522914761E-2,  .99959388468
			, Center.h, Center.v);
*/	
}
void Tourne(short i)
{
	rot.ApplyMatrix	(pts[i],Center);
}


efloatPoint eSetCenter(long time)
{
  static efloatPoint a(p.d1,0,0),c(0,0,0);
  matrix rot;

  if(p.mode>1)
    {
      rot=matrix(p.rotspeed2,efloatPoint(cos(((double)time)/40.)*5,cos(((double)time)/124.)*5,cos(((double)time)/231.)*5));
      for(int i=0;i<numCenters;i++)
	{
	    rot.ApplyMatrix (Centers[i],c);
	}
    }
  else
    {
      //  srand(time/100);
      rot=matrix(p.rotspeed2,efloatPoint(cos(((double)time)/40.)*5,cos(((double)time)/124.)*5,cos(((double)time)/231.)*5));
      rot.ApplyMatrix (a,c);

      efloatPoint dist=a;
      float d=(float)dist.length();
      float d0=p.d1;
      efloatPoint vectunite=(dist / d);
      float f=-1*(d-d0);
  
      vectunite = vectunite* f;
      a+= vectunite ;
    }

  return a;//efloatPoint(200+(rand()%(width-200)),200+(rand()%(height-200)),0);

}


void addpoints(char *p)
{
  int x,y;
  char *a,*b;
  efloatPoint t;
  while(1)
    {
      while(*p && *p!='(')p++;
      if(*p==0)
	return;
      p++;
      a=p;
      while(*p && *p!=',')p++;
      if(*p==0)
	return;

      *p=0;
      p++;
      b=p;
      while(*p && *p!=')')p++;
      if(*p==0)
	return;

      *p=0;
      p++;
      x=atoi(a);
      y=atoi(b);
      t=efloatPoint(x/15 - 300,y/15-200,0);
      if(newline[numCenters])
	{
	  Centers[numCenters++]=t;
	  newline[numCenters]=0;
	}
      else
	{
	  int last=numCenters-1;
	  for(int i=1;i<=numpart;i++)
	    {
	      Centers[numCenters++]= (Centers[last]*(numpart-i) + t*i )/numpart;
	      //printf(" %f %f %f \n", Centers[numCenters-1][0],Centers[numCenters-1][1],Centers[numCenters-1][2]);
	    }
	  newline[numCenters]=0;
	}

    }
  
    
}

// we should change this hack..
void loadepic2(char *file)
{
  FILE *f = fopen(file,"r");
  int mode=0;
  char buf[255];

  if(!f)
    printf("pas de fichier %s \n", file);

  numCenters = 0;
  fgets(buf,255,f);
  while(!feof(f))
    {

      if(buf[0]=='\\')
	mode = 0;
      switch(mode)
	{
	case 0:
	  if(memcmp(buf,"\\drawline",9)==0)
	  {
	    char *p=buf+9;
	    newline[numCenters]=1;
	    addpoints(p);
	    mode++;
	  }
	  break;
	case 1:
	  addpoints(buf);
	  break;

	     
	}
      fgets(buf,255,f);
    }
  if(!numCenters)
    printf("pas de points\n");

}

void loadepic(char *file)
{
  char filename[255];
  strncpy (filename, "/usr/local/share/dancingparticles/",255);
  strncat (filename, file,255);
  numpart=1;
  loadepic2(filename);
  numpart=ptsNum/numCenters;
  loadepic2(filename);


}

==> dancingparticles0.4.tgz: dancingparticles0.4/etoile.cc <==
#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include "etoile.h"
#include <cstdio>
#include <cstring>

Rect	windRect;

efloatPoint *pts=0;
efloatPoint *speed=0;
efloatPoint *Centers=0;
int         *newline=0;
efloatPoint *Centers2=0;
int         *newline2=0;
int numCenters;
int numCenters2;

efloatPoint Center(200.,200.,0);
int ptsNum=2000, ptsNumMax=2000;

unsigned short *gfrq[256];
int beat;
int beatcnt;
int thick, quiet;

int frames = 0;

matrix rot;


void etoileLoop(void)
{
  int frames2=frames;
  static int nextframes;
  frames++;
  if((beat==2  && (beatcnt%p.duration_b ==1 )) || frames > nextframes)
    {
      changep();
      nextframes = frames + p.duration_f;

      if(beat==2)
	beat--;
    }
  Center=eSetCenter(frames);

  rot=matrix(p.rotspeed1,efloatPoint(cos(((double)frames2)/120.)*2,cos(((double)frames2)/54.)*2,cos(((double)frames2)/231.)*2));

  for(int i=0;i<ptsNum;i++)
    {
      switch(p.mode)
	{
	case 1:
	  Elastantig(i,Center);
	  Tourne(i);
	  break;
	case 2:
	case 3:
	default:
	  Elastantig(i,Centers[i%numCenters]);
	  Tourne(i);
	}
    }
    	
}
void Elastantig(short i , efloatPoint & ctr)
{
  efloatPoint dist = pts[i]-ctr;
  float d=(float)dist.length();
  efloatPoint vectunite=(dist / d);
  
  // antigravitation  try not to make it n^2
  for(int k=0;k<p.agnumparts;k++)
    {
      int j = (frames + k + i )%ptsNum; 
      /*      if((i==0) &&(k==0))
	cout << j << " ";
      */
      if(i!=j)
	{
	  efloatPoint vect=pts[i]-pts[j];
	  float d1=(float)vect.length();
	  float d2=d1;
	  if(d2!=0)
	    {
	      // d include the normalisation
	      for(int k=0; k< p.antigorder;k++)
		d2=d2*d1;
	      d2=(10*p.ag)/d2;
	      
	      if(d2>p.maxantig)
		d2=p.maxantig;
	      vect=(vect * d2);
	      if(p.noagexplosion)// just to make a sphere
		vect= vect - ( vectunite * (vect % vectunite));

	      speed[i]+= vect ;
	    }
	  else
	    {
	      pts[i][0]+=0.01;
	    }
	}
    }

  float d0=p.d0min+ (((p.dancingpartk*i +frames)%ptsNum<p.dancingpart*ptsNum)?
		     (heights[(p.dancingpartk*i)%p.numfrq]*p.velocity)
		     :0);
  float f=-p.k*(d-d0);
  vectunite = vectunite* f;
  speed[i]+= vectunite ;
  

  speed[i]= speed[i]  /p.visc;
  pts[i]+= speed[i];// /1000;

}

void Tourne(short i)
{
	rot.ApplyMatrix	(pts[i],Center);
}


efloatPoint eSetCenter(long time)
{
  static efloatPoint a(p.d1,0,0),c(0,0,0);
  matrix rot;
  if(p.mode>1)
    {
      rot=matrix(p.rotspeed2,efloatPoint(cos(((double)time)/40.)*5,cos(((double)time)/124.)*5,cos(((double)time)/231.)*5));
      for(int i=0;i<numCenters;i++)
	{
	    rot.ApplyMatrix (Centers[i],c);
	}
    }
  else
    {
      //  srand(time/100);
      rot=matrix(p.rotspeed2,efloatPoint(cos(((double)time)/40.)*5,cos(((double)time)/124.)*5,cos(((double)time)/231.)*5));
      rot.ApplyMatrix (a,c);

      efloatPoint dist=a;
      float d=(float)dist.length();
      float d0=p.d1;
      efloatPoint vectunite=(dist / d);
      float f=-1*(d-d0);
  
      vectunite = vectunite* f;
      a+= vectunite ;
    }

  return a;//efloatPoint(200+(rand()%(width-200)),200+(rand()%(height-200)),0);

}
